"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const Contracts_1 = require("../Contracts");
const Emitter_1 = require("../Emitter");
const ow_1 = require("ow");
class Runner {
    constructor(_groups, _options) {
        this._groups = _groups;
        this._options = _options;
        this._hooks = {
            before: [],
            after: [],
        };
    }
    get hasErrors() {
        return !!this._groups.find((group) => group.hasErrors);
    }
    reporter(fn) {
        ow_1.default(fn, 'callback', ow_1.default.function);
        this._reporterFn = fn;
        return this;
    }
    before(fn) {
        ow_1.default(fn, 'callback', ow_1.default.function);
        this._hooks.before.push(fn);
        return this;
    }
    after(fn) {
        ow_1.default(fn, 'callback', ow_1.default.function);
        this._hooks.after.push(fn);
        return this;
    }
    async run() {
        if (typeof (this._reporterFn) !== 'function') {
            throw new Error('Make sure to define tests reporter as a function');
        }
        for (let hook of this._hooks.before) {
            await hook(this, Emitter_1.emitter);
        }
        this._reporterFn(Emitter_1.emitter, this._options);
        Emitter_1.emitter.emit(Contracts_1.IEvents.STARTED);
        for (let group of this._groups) {
            await group.run();
            if (this._options.bail && group.hasErrors) {
                break;
            }
        }
        Emitter_1.emitter.emit(Contracts_1.IEvents.COMPLETED);
        for (let hook of this._hooks.after) {
            await hook(this, Emitter_1.emitter);
        }
    }
}
exports.Runner = Runner;
